@@ -27,6 +27,7 @@
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
+#include <libhal.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <eel/eel-features.h>
@@ -258,6 +259,70 @@ copy_disc_activate_cb (NautilusMenuItem *item,
 	g_free (argv [0]);
 }
 
+
+static LibHalContext *
+get_hal_context (void)
+{
+	static LibHalContext *ctx = NULL;
+	DBusError	      error;
+	DBusConnection	     *dbus_conn;
+
+	if (ctx == NULL) {
+		ctx = libhal_ctx_new ();
+		if (ctx == NULL) {
+			g_warning ("Could not create a HAL context");
+		} else {
+			dbus_error_init (&error);
+			dbus_conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
+
+			if (dbus_error_is_set (&error)) {
+				g_warning ("Could not connect to system bus: %s", error.message);
+				dbus_error_free (&error);
+				return NULL;
+			}
+
+			libhal_ctx_set_dbus_connection (ctx, dbus_conn);
+
+			if (! libhal_ctx_init (ctx, &error)) {
+				g_warning ("Could not initalize the HAL context: %s",
+					   error.message);
+
+				if (dbus_error_is_set (&error))
+					dbus_error_free (&error);
+
+				libhal_ctx_free (ctx);
+				ctx = NULL;
+			}
+		}
+	}
+
+	return ctx;
+}
+
+static gboolean
+volume_is_blank (GnomeVFSVolume *volume)
+{
+	LibHalContext *ctx;
+	DBusError      error;
+	char          *udi;
+	gboolean       is_blank;
+
+	ctx = get_hal_context ();
+	if (ctx == NULL) {
+		return FALSE;
+	}
+
+	udi = gnome_vfs_volume_get_hal_udi (volume);
+	dbus_error_init (&error);
+	is_blank = libhal_device_get_property_bool (ctx,
+						    udi,
+						    "volume.disc.is_blank",
+						    NULL);
+	g_free (udi);
+
+	return is_blank;
+}
+
 static GList *
 nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 			      GtkWidget            *window,
@@ -269,7 +334,6 @@ nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 	GnomeVFSFileInfo *info;
 	GnomeVFSVolume   *volume;
 	GnomeVFSDrive    *drive;
-	char             *device_path;
 	char             *mime_type;
 	gboolean          is_local;
 	gboolean          is_iso;
@@ -326,16 +390,19 @@ nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 	}
 
 	drive = nautilus_file_info_get_drive (file);
-	if (drive == NULL) {
-		volume = nautilus_file_info_get_volume (file);
-		if (volume != NULL) {
-			drive = gnome_vfs_volume_get_drive (volume);
-			gnome_vfs_drive_unref (drive);
-		}
+	volume = nautilus_file_info_get_volume (file);
+	if (drive == NULL
+	    && volume != NULL) {
+		drive = gnome_vfs_volume_get_drive (volume);
+		gnome_vfs_drive_unref (drive);
 	}
 
 	if (drive != NULL
-	    && (gnome_vfs_drive_get_device_type (drive) == GNOME_VFS_DEVICE_TYPE_CDROM)) {
+	    && volume != NULL
+	    && (gnome_vfs_drive_get_device_type (drive) == GNOME_VFS_DEVICE_TYPE_CDROM)
+	    && !volume_is_blank (volume)) {
+		char *device_path;
+
 		device_path = gnome_vfs_drive_get_device_path (drive);
 
 		item = nautilus_menu_item_new ("NautilusBurn::copy_disc",
@@ -699,6 +766,11 @@ nautilus_module_initialize (GTypeModule *module)
 void
 nautilus_module_shutdown (void)
 {
+	LibHalContext *ctx;
+	ctx = get_hal_context ();
+	if (ctx != NULL) {
+		libhal_ctx_free (ctx);
+	}
 }
 
 void
